﻿//##############################################################################
//Licence
//##############################################################################
//Copyright (C) 2015  Christian Kreuzer (ch-kr@utanet.at)
//This program is free software; you can redistribute it and/or modify it
//under the terms of the GNU General Public License as published by the
//Free Software Foundation; either version 2 of the Licence.
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with
//this program; if not, write to the Free Software Foundation,
//Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
//
//##############################################################################
//Version
//##############################################################################
//V0.1	19.04.2015	Kreuzer		Grundversion
//V0.2  27.04.2015	Kreuzer		Schnittlinien hinzugefügt
//
//##############################################################################
//Funktionsbeschreibung
//##############################################################################
//Enthält alle nötigen Funktionen um einen Bild einen färbigen Rahmen und zusätzliche
//Schnittlinien hinzu zu fügen. Der färbige Rahmen wird aus den außersten Pixelreihen
//des Quellbildes erstellt und kann bei Bedarf verwischt werden.
//Bei dem Programm wurde nicht auf Performance wert gelegt.
//Zur Zeit werden nur Bilder mit identischer horizontaler und vertikaler Auflösung
//unterstützt.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.IO;


namespace JPG_TO_Canvas
{   //Event zum Ansteuern des Fortschittbalkens im Hauptfenster
    public delegate void PrograssEventEventHandler(object sender, ProgressEventArgs e);

    class Canvas
    {
        // Die Event für Fortschrittbalken
        public event PrograssEventEventHandler ProgressEvent;


        public enum verwischStufe
        {
            kein,
            schwach,
            mittel,
            stark
        }

        #region Variablen
        private Bitmap quellbild;
        private int zielWidth;
        private int zielHeight;
        private float zielHorPPI;
        private float zielVertPPI;
        private float zielHorDimension;
        private float zielVertDimension;
        private float zielRahmenDimension;
        private verwischStufe zielRahmenVerwischen;
        private float zielSchnittLinien;
        private int rahmenpixelGesamt;

        private Color FARBEsCHNITTLINIEN = Color.Black;
        #endregion

        #region Eigenschaften
        /// <summary>
        /// Breite des Zielbildes in Pixel
        /// </summary>
        public int ZielWidth
        {
            get { return zielWidth; }
            set 
            { 
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielWidth = value; 
            }
        }

        /// <summary>
        /// Höhe des Zielbildes in Pixel
        /// </summary>
        public int ZielHeight
        {
            get { return zielHeight; }
            set 
            {
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielHeight = value; 
            }
        }

        /// <summary>
        /// Horizontale Auflösung des Zielbildes
        /// </summary>
        public float ZielHorPPI
        {
            get { return zielHorPPI; }
            set 
            {
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielHorPPI = value;
                this.zielVertPPI = value;
                this.zielHorDimension = (float)this.quellbild.Width / this.zielHorPPI * (float)25.4;
                this.zielVertDimension = (float)this.quellbild.Height / this.zielVertPPI * (float)25.4;
                ZielbildPixelAktualisieren();
            }
        }

        /// <summary>
        /// Vertikale Auflösung des Zielbildes
        /// </summary>
        public float ZielVertPPI
        {
            get { return zielVertPPI; }
            set 
            {
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielHorPPI = value;
                this.zielVertPPI = value;
                this.zielHorDimension = (float)this.quellbild.Width / this.zielHorPPI * (float)25.4;
                this.zielVertDimension = (float)this.quellbild.Height / this.zielVertPPI * (float)25.4;
                ZielbildPixelAktualisieren();
            }
        }

        //Breite des Zielbildes in mm
        public float ZielHorDimension
        {
            get { return zielHorDimension; }
            set 
            {
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielHorDimension = value;
                this.zielHorPPI = (float)this.quellbild.Width * (float)25.4 / this.zielHorDimension;
                this.zielVertPPI = this.zielHorPPI;
                this.zielVertDimension = (float)this.quellbild.Height / this.zielVertPPI * (float)25.4;
                ZielbildPixelAktualisieren();
            }
        }

        //Höhe des Zielbildes in mm
        public float ZielVertDimension
        {
            get { return zielVertDimension; }
            set
            {
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielVertDimension = value;
                this.zielVertPPI = (float)this.quellbild.Height / this.zielVertDimension * (float)25.4;
                this.zielHorPPI = this.zielVertPPI;
                this.zielHorDimension = (float)this.quellbild.Width / this.zielHorPPI * (float)25.4;
                ZielbildPixelAktualisieren();
            }
        }

        //Breite des färbigen Rahmens in mm
        public float ZielRahmenDimension
        {
            get { return zielRahmenDimension; }
            set 
            {
                if (value <= 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielRahmenDimension = value;
                ZielbildPixelAktualisieren();
            }
        }

        /// <summary>
        /// Stärke des Verwischens des Färbigen Rahmens
        /// </summary>
        public verwischStufe ZielRahmenVerwischen
        {
            get { return zielRahmenVerwischen; }
            set { zielRahmenVerwischen = value; }
        }

        /// <summary>
        /// Abstand der Schnittlinien zum Rand des Quellbildes
        /// </summary>
        public float ZielSchnittLinien
        {
            get { return zielSchnittLinien; }
            set
            {
                if (value < 0)
                {
                    throw new ArgumentOutOfRangeException("Werte muss positiv sein!");
                }
                this.zielSchnittLinien = value;
                ZielbildPixelAktualisieren();
            }
        }

        /// <summary>
        /// Berechnet den mindest Betrachtungsabstand des Bildes und zeigt ihn an
        /// </summary>
        public int MinBetrachtungsabstand
        {
            get 
            {
                return (int)(5500 / this.ZielHorPPI);
            }
        }
        #endregion

        #region Konstruktor
        /// <summary>
        /// Erstellt eine neue Instanz
        /// </summary>
        /// <param name="quellb">Quellbild als Bitmap</param>
        public Canvas(Bitmap quellb)
        {
            this.quellbild = quellb;
            QuellbildToEigenschaften();
        }

        /// <summary>
        /// Erstellt eine neue Instanz
        /// </summary>
        /// <param name="pfad">Pfad des zu lesenden Quellbildes</param>
        public Canvas(string pfad)
        {
            if (File.Exists(pfad) == false)
            {
                throw new FileNotFoundException();
            }
            this.quellbild = (Bitmap)Image.FromFile(pfad);
            QuellbildToEigenschaften();
        }
        #endregion

        #region Methoden
        /// <summary>
        /// Erstellt ein neues Bitmap, kopiert das Quellbild in die Mitte und fügt den Rahmen hinzu
        /// </summary>
        /// <returns>Bild mit Rahmen</returns>
        public Bitmap RahmenHinzufuegen()
        {
            int randpixel = (int)(this.zielRahmenDimension * this.zielHorPPI / 25.4);
            int schnittlinienpixel = this.rahmenpixelGesamt - randpixel;
            int aktZeile = 0;

            //erstellt Zielbild mit weißen Hintergrund
            Bitmap zielbild = new Bitmap(this.zielWidth, this.zielHeight);
            Rectangle quellBereich = new Rectangle(0, 0, 1, 1);
            Rectangle zielBereich = new Rectangle(0, 0, this.zielWidth, this.zielHeight);
            if (schnittlinienpixel > 0)
            {
                using (Graphics grD = Graphics.FromImage(zielbild))
                {
                    grD.FillRectangle(new SolidBrush(Color.White), zielBereich);
                }
            }

            //kopiert das Quellbild in die Mitte des Zielbildes
            quellBereich = new Rectangle(0, 0, this.quellbild.Width, this.quellbild.Height);
            zielBereich = new Rectangle((this.zielWidth - this.quellbild.Width) / 2, (this.zielHeight - this.quellbild.Height) / 2, this.quellbild.Width, this.quellbild.Height);
            using (Graphics grD = Graphics.FromImage(zielbild))
            {
                grD.DrawImage(this.quellbild, zielBereich, quellBereich, GraphicsUnit.Pixel);
            }

            zielbild.SetResolution(this.zielHorPPI, this.zielVertPPI);

            //Verwischen
            int verwischBreite = 0;
            if (this.zielRahmenVerwischen != verwischStufe.kein)
            {
                verwischBreite = (2 * (int)this.zielRahmenVerwischen) + 1;
            }

            OnProgressEvent((4 * randpixel), aktZeile);

            #region Ecken erweitern
            //Innerhalb des Rahmens mit den jeweiligen Farben der Eckpixel des Quellbildes
            //Innerhalb der Schnittlinien mit schwarzen Linien
            //oben
            int yZiel = this.rahmenpixelGesamt;
            for (int xZ = schnittlinienpixel; xZ < (zielbild.Width - schnittlinienpixel); xZ++)
            {
                //linker Rand
                if (xZ < this.rahmenpixelGesamt)
                {
                    zielbild.SetPixel(xZ, yZiel, this.quellbild.GetPixel(0, 0));
                }
                //rechter Rand
                else if (xZ >= (this.rahmenpixelGesamt + this.quellbild.Width))
                {
                    zielbild.SetPixel(xZ, yZiel, this.quellbild.GetPixel(this.quellbild.Width - 1, 0));
                }
                //Bildbreite
                else
                {
                    xZ = this.rahmenpixelGesamt + this.quellbild.Width - 1;
                }
            }

            //unten
            yZiel = this.rahmenpixelGesamt + this.quellbild.Height - 1;
            for (int xZ = schnittlinienpixel; xZ < (zielbild.Width - schnittlinienpixel); xZ++)
            {
                //linker Rand
                if (xZ < this.rahmenpixelGesamt)
                {
                    zielbild.SetPixel(xZ, yZiel, this.quellbild.GetPixel(0, this.quellbild.Height - 1));
                }
                //rechter Rand
                else if (xZ >= (this.rahmenpixelGesamt + this.quellbild.Width))
                {
                    zielbild.SetPixel(xZ, yZiel, this.quellbild.GetPixel(this.quellbild.Width - 1, this.quellbild.Height - 1));
                }
                //Bildbreite
                else
                {
                    xZ = this.rahmenpixelGesamt + this.quellbild.Width - 1;
                }
            }
            #endregion

            #region färbigen Rahmen hinzufügen
            //oberer Rand
            for (int yZ = (this.rahmenpixelGesamt - 1); yZ >= schnittlinienpixel; yZ--)
            {
                for (int xZ = schnittlinienpixel; xZ < (zielbild.Width - schnittlinienpixel); xZ++)
                {
                    //linker Rand
                    if (xZ < verwischBreite)
                    {
                        zielbild.SetPixel(xZ, yZ, this.quellbild.GetPixel(0, 0));
                    }
                    //rechter Rand
                    else if (xZ >= (zielbild.Width - verwischBreite - 1))
                    {
                        zielbild.SetPixel(xZ, yZ, this.quellbild.GetPixel(this.quellbild.Width - 1, 0));
                    }
                    //Bildbreite
                    else
                    {
                        if (yZ > (this.rahmenpixelGesamt - verwischBreite))  //Verwischen
                        {
                            int[] farben = new int[3];
                            for (int i = ((verwischBreite - 1) / -2); i < ((verwischBreite + 1) / 2); i++)
                            {
                                farben[0] += zielbild.GetPixel(xZ + i, yZ + 1).R;
                                farben[1] += zielbild.GetPixel(xZ + i, yZ + 1).G;
                                farben[2] += zielbild.GetPixel(xZ + i, yZ + 1).B;
                            }
                            farben[0] /= verwischBreite;
                            farben[1] /= verwischBreite;
                            farben[2] /= verwischBreite;

                            zielbild.SetPixel(xZ, yZ, Color.FromArgb(farben[0], farben[1], farben[2]));
                        }
                        else
                        {
                            zielbild.SetPixel(xZ, yZ, zielbild.GetPixel(xZ, yZ + 1));
                        }
                    }
                }
                OnProgressEvent((4 * randpixel), ++aktZeile);
            }

            //unterer Rand
            for (int yZ = (this.rahmenpixelGesamt + this.quellbild.Height); yZ < (zielbild.Height - schnittlinienpixel); yZ++)
            {
                for (int xZ = schnittlinienpixel; xZ < (zielbild.Width - schnittlinienpixel); xZ++)
                {
                    //linker Rand
                    if (xZ < verwischBreite)
                    {
                        zielbild.SetPixel(xZ, yZ, this.quellbild.GetPixel(0, this.quellbild.Height - 1));
                    }
                    //rechter Rand
                    else if (xZ >= (zielbild.Width - verwischBreite - 1))
                    {
                        zielbild.SetPixel(xZ, yZ, this.quellbild.GetPixel(this.quellbild.Width - 1, this.quellbild.Height - 1));
                    }
                    //Bildbreite
                    else
                    {
                        if (yZ < (this.rahmenpixelGesamt + this.quellbild.Height + verwischBreite))  //Verwischen
                        {
                            int[] farben = new int[3];
                            for (int i = ((verwischBreite - 1) / -2); i < ((verwischBreite + 1) / 2); i++)
                            {
                                farben[0] += zielbild.GetPixel(xZ + i, yZ - 1).R;
                                farben[1] += zielbild.GetPixel(xZ + i, yZ - 1).G;
                                farben[2] += zielbild.GetPixel(xZ + i, yZ - 1).B;
                            }
                            farben[0] /= verwischBreite;
                            farben[1] /= verwischBreite;
                            farben[2] /= verwischBreite;

                            zielbild.SetPixel(xZ, yZ, Color.FromArgb(farben[0], farben[1], farben[2]));
                        }
                        else
                        {
                            zielbild.SetPixel(xZ, yZ, zielbild.GetPixel(xZ, yZ - 1));
                        }
                    }
                }
                OnProgressEvent((4 * randpixel), ++aktZeile);
            }

            //linker Rand
            for (int xZ = (this.rahmenpixelGesamt - 1); xZ >= schnittlinienpixel; xZ--)
            {
                for (int yZ = this.rahmenpixelGesamt; yZ < (zielbild.Height - this.rahmenpixelGesamt); yZ++)
                {
                    if (xZ > (this.rahmenpixelGesamt - verwischBreite))  //Verwischen
                    {
                        int[] farben = new int[3];
                        for (int i = ((verwischBreite - 1) / -2); i < ((verwischBreite + 1) / 2); i++)
                        {
                            farben[0] += zielbild.GetPixel(xZ + 1, yZ + i).R;
                            farben[1] += zielbild.GetPixel(xZ + 1, yZ + i).G;
                            farben[2] += zielbild.GetPixel(xZ + 1, yZ + i).B;
                        }
                        farben[0] /= verwischBreite;
                        farben[1] /= verwischBreite;
                        farben[2] /= verwischBreite;

                        zielbild.SetPixel(xZ, yZ, Color.FromArgb(farben[0], farben[1], farben[2]));
                    }
                    else
                    {
                        zielbild.SetPixel(xZ, yZ, zielbild.GetPixel(xZ + 1, yZ));
                    }
                }
                OnProgressEvent((4 * randpixel), ++aktZeile);
            }

            //rechter Rand
            for (int xZ = (this.rahmenpixelGesamt + this.quellbild.Width); xZ < (zielbild.Width - schnittlinienpixel); xZ++)
            {
                for (int yZ = this.rahmenpixelGesamt; yZ < (zielbild.Height - this.rahmenpixelGesamt); yZ++)
                {
                    if (xZ < (this.rahmenpixelGesamt + this.quellbild.Width + verwischBreite))  //Verwischen
                    {
                        int[] farben = new int[3];
                        for (int i = ((verwischBreite - 1) / -2); i < ((verwischBreite + 1) / 2); i++)
                        {
                            farben[0] += zielbild.GetPixel(xZ - 1, yZ + i).R;
                            farben[1] += zielbild.GetPixel(xZ - 1, yZ + i).G;
                            farben[2] += zielbild.GetPixel(xZ - 1, yZ + i).B;
                        }
                        farben[0] /= verwischBreite;
                        farben[1] /= verwischBreite;
                        farben[2] /= verwischBreite;

                        zielbild.SetPixel(xZ, yZ, Color.FromArgb(farben[0], farben[1], farben[2]));
                    }
                    else
                    {
                        zielbild.SetPixel(xZ, yZ, zielbild.GetPixel(xZ - 1, yZ));
                    }
                }
                OnProgressEvent((4 * randpixel), ++aktZeile);
            }
            #endregion

            #region Schnittlinien hinzufügen
            if (schnittlinienpixel > 0)
            {
                //Außen Rahmen
                quellBereich = new Rectangle(0, 0, 1, 1);
                zielBereich = new Rectangle(0, 0, this.zielWidth - 1, this.zielHeight - 1);
                using (Graphics grD = Graphics.FromImage(zielbild))
                {
                    grD.DrawRectangle(new Pen(FARBEsCHNITTLINIEN), zielBereich);
                }

                //Verlängerung der Bildkanten
                for (int yZ = 0; yZ < schnittlinienpixel; yZ++)
                {
                    zielbild.SetPixel(this.rahmenpixelGesamt, yZ, FARBEsCHNITTLINIEN);
                    zielbild.SetPixel((this.zielWidth - this.rahmenpixelGesamt), yZ, FARBEsCHNITTLINIEN);
                }
                for (int yZ = (this.zielHeight - schnittlinienpixel); yZ < this.zielHeight; yZ++)
                {
                    zielbild.SetPixel(this.rahmenpixelGesamt, yZ, FARBEsCHNITTLINIEN);
                    zielbild.SetPixel((this.zielWidth - this.rahmenpixelGesamt), yZ, FARBEsCHNITTLINIEN);
                }
                for (int xZ = 0; xZ < schnittlinienpixel; xZ++)
                {
                    zielbild.SetPixel(xZ, this.rahmenpixelGesamt, FARBEsCHNITTLINIEN);
                    zielbild.SetPixel(xZ, (this.zielHeight - this.rahmenpixelGesamt), FARBEsCHNITTLINIEN);
                }
                for (int xZ = (this.zielWidth - schnittlinienpixel); xZ < this.zielWidth; xZ++)
                {
                    zielbild.SetPixel(xZ, this.rahmenpixelGesamt, FARBEsCHNITTLINIEN);
                    zielbild.SetPixel(xZ, (this.zielHeight - this.rahmenpixelGesamt), FARBEsCHNITTLINIEN);
                }
            }
            #endregion
            return zielbild;
        }
        #endregion

        #region private Methoden
        /// <summary>
        /// Wird vom Konstruktor aufgerufen und errechnet die Eigenschaftes des Zielbildes aus
        /// dem übergebenen Quellbild
        /// </summary>
        private void QuellbildToEigenschaften()
        {
            this.zielWidth = this.quellbild.Width;
            this.zielHeight = this.quellbild.Height;
            this.zielHorPPI = this.quellbild.HorizontalResolution;
            this.zielVertPPI = this.quellbild.VerticalResolution;
            this.zielHorDimension = (float)this.zielWidth / this.zielHorPPI * (float)25.4;
            this.zielVertDimension = (float)this.zielHeight / this.zielVertPPI * (float)25.4;
        }

        /// <summary>
        /// Errechnet die Gesamtgröße des Bildes in Pixel (fügt Rahmengröße hinzu)
        /// </summary>
        private void ZielbildPixelAktualisieren()
        {
            if (this.zielRahmenDimension > this.zielSchnittLinien)
            {
                this.rahmenpixelGesamt = (int)(this.zielRahmenDimension * this.zielHorPPI / 25.4);
            }
            else
            {
                this.rahmenpixelGesamt = (int)(this.zielSchnittLinien * this.zielHorPPI / 25.4);
            }
            this.zielWidth = 2 * this.rahmenpixelGesamt + this.quellbild.Width;
            this.zielHeight = 2 * this.rahmenpixelGesamt + this.quellbild.Height;
        }


        // Eine private Methode zum Auslösen des Events. Vorher prüfen, ob auch
        // ein Empfänger an dem Event angemeldet ist.
        private void OnProgressEvent(int max, int actual)
        {
            if (ProgressEvent != null)
                ProgressEvent(this, new ProgressEventArgs(max, actual));
        }

        #endregion


    }

    #region Klasse EventArgs
    /// <summary>
    /// Klasse für die Übergabe des Events an das Hauptfenster
    /// </summary>
    public class ProgressEventArgs : EventArgs
    {
        private int _maxValue = 0;
        private int _actualValue = 0;

        public ProgressEventArgs(int max, int actual)
        {
            _maxValue = max;
            _actualValue = actual;
        }

        /// <summary>
        /// Maximaler Wert der Prograssbar
        /// </summary>
        public int MaxValue
        {
            get { return _maxValue; }
            set { _maxValue = value; }
        }

        /// <summary>
        /// Aktueller Wert der Prograssbar
        /// </summary>
        public int ActualValue
        {
            get { return _actualValue; }
            set { _actualValue = value; }
        }
    }
    #endregion
}
