﻿//##############################################################################
//Licence
//##############################################################################
//Copyright (C) 2015  Christian Kreuzer (ch-kr@utanet.at)
//This program is free software; you can redistribute it and/or modify it
//under the terms of the GNU General Public License as published by the
//Free Software Foundation; either version 2 of the Licence.
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with
//this program; if not, write to the Free Software Foundation,
//Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
//
//##############################################################################
//Version
//##############################################################################
//V0.1	19.04.2015	Kreuzer		Grundversion
//V0.2  27.04.2015	Kreuzer		Schnittlinien hinzugefügt
//
//##############################################################################
//Funktionsbeschreibung
//##############################################################################

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace JPG_TO_Canvas
{
    public partial class Hauptfenster : Form
    {
        #region Variablen und Konstanten
        private const float RAHMENBREITE = 30;
        private const float SCHNITTLINIEN = 55;
        Canvas canvasBild;
        #endregion

        #region Konstruktor
        public Hauptfenster()
        {
            InitializeComponent();
            cobo_Verwischen.DataSource = Enum.GetValues(typeof(Canvas.verwischStufe));
            string programmname = ((System.Reflection.AssemblyProductAttribute)System.Reflection.Assembly.GetEntryAssembly().GetCustomAttributes(typeof(System.Reflection.AssemblyProductAttribute), false)[0]).Product;
            string programmversion = ((System.Reflection.AssemblyFileVersionAttribute)System.Reflection.Assembly.GetEntryAssembly().GetCustomAttributes(typeof(System.Reflection.AssemblyFileVersionAttribute), false)[0]).Version;
            string copyright = ((System.Reflection.AssemblyCopyrightAttribute)System.Reflection.Assembly.GetEntryAssembly().GetCustomAttributes(typeof(System.Reflection.AssemblyCopyrightAttribute), false)[0]).Copyright;
            this.Text = programmname + " V" + programmversion + " " + copyright;
            //Tooltip
            toolTip1.SetToolTip(this.btnQuellpfad, "Öffnet die Dateiauswahl für das Quellbild.");
            toolTip1.SetToolTip(this.tbxRahmenMm, "Breite des färbigen Rahmens welcher um das Quellbild erstellt wird in mm.");
            toolTip1.SetToolTip(this.tbxSchnittlinienMm, "Abstand in mm um das Quellbild zum Hinzufügen der Schnittlinien.\r\nWenn kleiner der Rahmenbreite werden keine Schnittlinien hinzugefügt.");
            toolTip1.SetToolTip(this.tbxBetrachtungsAbstand, "Empfohlener Mindestbetrachtungsabstand in cm, damit die einzelnen Pixel nicht erkannt werden.");
            toolTip1.SetToolTip(this.btnKonvert, "Startet die Konvertierung.");
        }
        #endregion

        #region Benutzereingaben
        /// <summary>
        /// Öffnet das ausgewählte Bild und speichert es als quellbild.
        /// Füllt die Werte im Container Quellbild und schaltet die Zieloptionen frei.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnQuellpfad_Click(object sender, EventArgs e)
        {
            OpenFileDialog dateiOeffnen = new OpenFileDialog();
            dateiOeffnen.Title = "Quellbild öffnen";
            dateiOeffnen.Multiselect = false;
            dateiOeffnen.DefaultExt = "bmp";
            dateiOeffnen.Filter = "Windows JPG (*.jpg)|*.jpg";
            dateiOeffnen.ShowDialog();
            if (dateiOeffnen.FileName.Length > 0)
            {
                tbxQuellpfad.TextAlign = HorizontalAlignment.Left;
                tbxQuellpfad.Text = dateiOeffnen.FileName;
                try
                {
                    canvasBild = new Canvas(dateiOeffnen.FileName);
                    canvasBild.ProgressEvent += new PrograssEventEventHandler(canvasBild_ProgressEvent);

                    tbxHeightPixel.Text = canvasBild.ZielHeight.ToString();
                    tbxHeightResolution.Text = canvasBild.ZielVertPPI.ToString();
                    tbxHeigthMm.Text = canvasBild.ZielVertDimension.ToString();

                    tbxWidthPixel.Text = canvasBild.ZielWidth.ToString();
                    tbxWidthResolution.Text = canvasBild.ZielHorPPI.ToString();
                    tbxWidthMm.Text = canvasBild.ZielHorDimension.ToString();

                    tbxRahmenMm.Text = RAHMENBREITE.ToString();
                    canvasBild.ZielRahmenDimension = RAHMENBREITE;

                    tbxSchnittlinienMm.Text = SCHNITTLINIEN.ToString();
                    canvasBild.ZielSchnittLinien = SCHNITTLINIEN;

                    ZielanzeigenAktualisieren();

                    grbEigenschaften.Visible = true;
                    btnKonvert.Enabled = true;
                }
                catch (Exception ee)
                {
                    MessageBox.Show(ee.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Errechnet die geänderten Zieloptionen und zeigt sie an.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbxHeightResolutionZiel_Validating(object sender, CancelEventArgs e)
        {
            if (IsNotPosFloat(tbxHeightResolutionZiel.Text))
            {
                e.Cancel = true;
                tbxHeightResolutionZiel.Select(0, tbxHeightResolutionZiel.Text.Length);
                tbxHeightResolutionZiel.BackColor = System.Drawing.Color.Red;
            }
            else
            {
                tbxHeightResolutionZiel.BackColor = System.Drawing.SystemColors.Window;
                canvasBild.ZielVertPPI  = float.Parse(tbxHeightResolutionZiel.Text);
                ZielanzeigenAktualisieren();
            }
        }

        /// <summary>
        /// Errechnet die geänderten Zieloptionen und zeigt sie an.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbxHeigthMmZiel_Validating(object sender, CancelEventArgs e)
        {
            if (IsNotPosFloat(tbxHeightMmZiel.Text))
            {
                e.Cancel = true;
                tbxHeightMmZiel.Select(0, tbxHeightMmZiel.Text.Length);
                tbxHeightMmZiel.BackColor = System.Drawing.Color.Red;
            }
            else
            {
                tbxHeightMmZiel.BackColor = System.Drawing.SystemColors.Window;
                canvasBild.ZielVertDimension = float.Parse(tbxHeightMmZiel.Text);
                ZielanzeigenAktualisieren();
            }
        }

        /// <summary>
        /// Errechnet die geänderten Zieloptionen und zeigt sie an.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbxWidthResolutionZiel_Validating(object sender, CancelEventArgs e)
        {
            if (IsNotPosFloat(tbxWidthResolutionZiel.Text))
            {
                e.Cancel = true;
                tbxWidthResolutionZiel.Select(0, tbxWidthResolutionZiel.Text.Length);
                tbxWidthResolutionZiel.BackColor = System.Drawing.Color.Red;
            }
            else
            {
                tbxWidthResolutionZiel.BackColor = System.Drawing.SystemColors.Window;
                canvasBild.ZielHorPPI = float.Parse(tbxWidthResolutionZiel.Text);
                ZielanzeigenAktualisieren();
            }
        }


        /// <summary>
        /// Errechnet die geänderten Zieloptionen und zeigt sie an.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbxWidthMmZiel_Validating(object sender, CancelEventArgs e)
        {
            if (IsNotPosFloat(tbxWidthMmZiel.Text))
            {
                e.Cancel = true;
                tbxWidthMmZiel.Select(0, tbxWidthMmZiel.Text.Length);
                tbxWidthMmZiel.BackColor = System.Drawing.Color.Red;
            }
            else
            {
                tbxWidthMmZiel.BackColor = System.Drawing.SystemColors.Window;
                canvasBild.ZielHorDimension = float.Parse(tbxWidthMmZiel.Text);
                ZielanzeigenAktualisieren();
            }
        }

        /// <summary>
        /// Errechnet die geänderten Zieloptionen und zeigt sie an.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbxRahmenMm_Validating(object sender, CancelEventArgs e)
        {
            if (IsNotPosFloat(tbxRahmenMm.Text))
            {
                e.Cancel = true;
                tbxRahmenMm.Select(0, tbxRahmenMm.Text.Length);
                tbxRahmenMm.BackColor = System.Drawing.Color.Red;
            }
            else
            {
                tbxRahmenMm.BackColor = System.Drawing.SystemColors.Window;
                canvasBild.ZielRahmenDimension = float.Parse(tbxRahmenMm.Text);
                ZielanzeigenAktualisieren();
            }
        }

        /// <summary>
        /// Errechnet die geänderten Zieloptionen und zeigt sie an.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbxSchnittlinienMm_Validating(object sender, CancelEventArgs e)
        {
            if (IsNotPosFloat(tbxSchnittlinienMm.Text))
            {
                if (tbxSchnittlinienMm.Text != "")
                {
                    e.Cancel = true;
                    tbxSchnittlinienMm.Select(0, tbxSchnittlinienMm.Text.Length);
                    tbxSchnittlinienMm.BackColor = System.Drawing.Color.Red;
                }
                else
                {
                    canvasBild.ZielSchnittLinien = 0;
                }
            }
            else
            {
                tbxSchnittlinienMm.BackColor = System.Drawing.SystemColors.Window;
                canvasBild.ZielSchnittLinien = float.Parse(tbxSchnittlinienMm.Text);
                ZielanzeigenAktualisieren();
            }
        }


        /// <summary>
        /// Konvertiert und speichert das Zielbild
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnKonvert_Click(object sender, EventArgs e)
        {
            canvasBild.ZielRahmenVerwischen = (Canvas.verwischStufe)cobo_Verwischen.SelectedItem;

            string zielpfad = tbxQuellpfad.Text;
            zielpfad = zielpfad.Remove(zielpfad.LastIndexOf('.'));
            zielpfad += ".bmp";

            progB_Konvert.Visible = true;
            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;

            Bitmap ergebnis = canvasBild.RahmenHinzufuegen();
            ergebnis.Save(zielpfad, System.Drawing.Imaging.ImageFormat.Bmp);

            progB_Konvert.Visible = false;
            lbZieldatei.Text = zielpfad;
            lbZieldatei.Visible = true;
            this.Cursor = System.Windows.Forms.Cursors.Default;
        }
        #endregion

        #region private Methoden
        /// <summary>
        /// Parst einen String in ein Float
        /// Überpüft üb der Wert negativ oder kein Float ist
        /// </summary>
        /// <param name="text">String zu Parsen</param>
        /// <returns>false wenn positiver Float Wert</returns>
        private bool IsNotPosFloat(string text)
        {
            float neuerWert;
            bool error = false;

            if (!float.TryParse(text, out neuerWert))
            {
                error = true;
            }
            if (error == false)
            {
                if (neuerWert < 0)
                {
                    error = true;
                }
            }
            return error;
        }

        /// <summary>
        /// Aktualisiert die Anzeigen mit den Eigenschaften des Zielbilds
        /// </summary>
        private void ZielanzeigenAktualisieren()
        {
            tbxHeightResolutionZiel.Text = string.Format("{0:0.##}", canvasBild.ZielVertPPI);
            tbxHeightMmZiel.Text = string.Format("{0:0.##}", canvasBild.ZielVertDimension);
            tbxHeightPixelZiel.Text = string.Format("{0:0.##}", canvasBild.ZielHeight);

            tbxWidthResolutionZiel.Text = string.Format("{0:0.##}", canvasBild.ZielHorPPI);
            tbxWidthMmZiel.Text = string.Format("{0:0.##}", canvasBild.ZielHorDimension);
            tbxWidthPixelZiel.Text = string.Format("{0:0.##}", canvasBild.ZielWidth);

            tbxBetrachtungsAbstand.Text = string.Format("{0:0.##}", canvasBild.MinBetrachtungsabstand);
        }

        /// <summary>
        /// Eventhandler für Progressbar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void canvasBild_ProgressEvent(object sender, ProgressEventArgs e)
        {
            progB_Konvert.Maximum = e.MaxValue;
            progB_Konvert.Value = e.ActualValue;
        }
        
        #endregion
    }
}
